from mecher_amt_api_client.client.endpoints.app_artefact import AppArtefact_Api
from mecher_amt_api_client.client.endpoints.app_run_config import AppRunConfig_Api
from mecher_amt_api_client.client.endpoints.auth import Auth_Api
from mecher_amt_api_client.client.endpoints.bs_owner import BSOwner_Api
from mecher_amt_api_client.models.amt_user import AMT_User_Auth


class AMT_Api(
    AppArtefact_Api,
    AppRunConfig_Api,
    BSOwner_Api,
    Auth_Api,
):
    def __init__(self, user: AMT_User_Auth):
        super().__init__()

        try:
            login_response = self.login(user)
            headers = {f"Authorization": f"Bearer {login_response['access_token']}"}
            self.default_headers_set = self._build_request_headers(headers)
        except KeyError:
            raise Exception("Login failed. Please check your credentials.")



from mecher_base_api_client.api_client import API


class AMT_ApiCommon(API):
    def __init__(self):
        super().__init__(url='http://0.0.0.0:8080/api',
                         default_status_codes=(200, 201, 204),
                         timeout=100)

    def send_request(self, uri: str, **kwargs):
        response = self.send_request_base(url=uri, **kwargs)
        return response.json()



import re
from typing import Optional

from fastapi import UploadFile, HTTPException
from pydantic import BaseModel
from starlette import status

from mecher_amt_api_client.models.app_run_config import DEFAULT_APP_FORMAT
from tests.constants import DEBUG_TAG_FOR_UNIT_TESTS


class AppArtefact__Upload(BaseModel):
    bs_owner: str
    project: str
    version: str = DEFAULT_APP_FORMAT
    tag: Optional[str] = ''

    platform: Optional[str]
    extension: Optional[str]
    file_name: Optional[str]

    def build_file_name(self):
        if self.tag == DEBUG_TAG_FOR_UNIT_TESTS:  # we need this workaround for unit tests
            self.file_name = DEBUG_TAG_FOR_UNIT_TESTS
        else:
            self.file_name = f'{self.project}__{self.bs_owner}__{self.platform}__{self.version}__{self.tag}.{self.extension}'


class AppArtefact__With_File(AppArtefact__Upload):
    file: UploadFile

    def determine_platform(self):
        if self.file.filename.endswith('.apk'):
            self.extension = 'apk'
            self.platform = 'android'
        elif self.file.filename.endswith('.ipa'):
            self.extension = 'ipa'
            self.platform = 'ios'
        else:
            raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST,
                                detail=f'Unknown file extension. Only .apk and .ipa are allowed: {self.file.filename}')

    def check_version_format(self):
        pattern = r'^v\d+_\d+_\d+$'
        if not re.match(pattern, DEFAULT_APP_FORMAT):
            raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST,
                                detail=f'Invalid re-expression: {pattern} for expected format: {DEFAULT_APP_FORMAT}')

        if not re.match(pattern, self.version):
            raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST,
                                detail=f'Invalid version format: {self.version}. Expected format: {DEFAULT_APP_FORMAT}')


class AppArtefact__Read(AppArtefact__Upload):
    bs_app_url: str
    bs_app_name: str

    class Config:
        openapi_example = {
            "example": {
                "summary": "example_uploaded_app",
                "value": {
                    "bs_owner": "mecher",
                    "project": "project",
                    "version": DEFAULT_APP_FORMAT,
                    "tag": "",
                    "platform": "ios",
                    "extension": "ipa",
                    "file_name": "file_name.ipa",
                    "bs_app_url": "https://app-url.com",
                    "bs_app_name": "app_name",
                }
            }
        }

    @staticmethod
    def default_instance() -> 'AppArtefact__Read':
        return AppArtefact__Read(**AppArtefact__Read.Config.openapi_example['example']['value'])


class AppArtefact__Search(BaseModel):
    project: str
    bs_owner: str
    platform: str
    version: str = DEFAULT_APP_FORMAT
    tag: str = ''

    class Config:
        openapi_example = {
            "example": {
                "summary": "example_app_for_search",
                "value": {
                    "project": "project",
                    "bs_owner": "mecher",
                    "platform": "ios",
                    "version": DEFAULT_APP_FORMAT,
                    "tag": ""
                }
            }
        }

    @staticmethod
    def default_instance() -> 'AppArtefact__Search':
        return AppArtefact__Search(**AppArtefact__Search.Config.openapi_example['example']['value'])



from pydantic import BaseModel


class BrowserstackOwner(BaseModel):
    user_name: str
    access_key: str
    bs_owner: str = 'mecher'

    class Config:
        openapi_example = {
            "example": {
                "summary": "example_bs_owner",
                "value": {
                    "user_name": "example_user",
                    "access_key": "example_key",
                    "bs_owner": "mecher"
                }
            }
        }

    @staticmethod
    def default_instance() -> 'BrowserstackOwner':
        return BrowserstackOwner(**BrowserstackOwner.Config.openapi_example['example_user']['value'])



from enum import Enum

from pydantic import BaseModel


class Roles(Enum):
    ADMIN = 'admin'
    MORTAL = 'mortal'

    @staticmethod
    def get_all_roles():
        return [role for role in Roles]


class AMT_User_Auth(BaseModel):
    username: str
    password: str

    class Config:
        openapi_example = {
            "example": {
                "summary": "example_user",
                "value": {
                    "username": "mecher",
                    "password": "123"
                }
            }
        }

    @staticmethod
    def default_instance() -> 'AMT_User_Auth':
        return AMT_User_Auth(**AMT_User_Auth.Config.openapi_example['example']['value'])


class AMT_User(AMT_User_Auth):
    role: Roles
    enabled: bool = True

    class Config:
        openapi_example = {
            "example": {
                "summary": "example_user",
                "value": {
                    **AMT_User_Auth.Config.openapi_example['example']['value'],
                    "role": Roles.MORTAL,
                    "enabled": True
                }
            }
        }

    @staticmethod
    def default_instance() -> 'AMT_User':
        return AMT_User(**AMT_User.Config.openapi_example['example']['value'])


class UserToken(BaseModel):
    username: str
    expires: int

    class Config:
        openapi_example = {
            "example": {
                "summary": "example_user",
                "value": {
                    "username": "username",
                    "expires": 161354390
                }
            }
        }

    @staticmethod
    def default_instance() -> 'UserToken':
        return UserToken(**UserToken.Config.openapi_example['example']['value'])



import allure

from mecher_amt_api_client.amt_api_common import AMT_ApiCommon
from mecher_amt_api_client.models.amt_user import AMT_User_Auth


class Auth_Api(AMT_ApiCommon):
    def __init__(self):
        super().__init__()
        self.auth_url = self.url + "/auth"

    def login(self, user: AMT_User_Auth):
        uri = self.auth_url + "/login"
        with allure.step('AMT API: Login'):
            response = self.send_request(method='post',
                                         body=user.dict(),
                                         uri=uri)
            return response



import allure

from mecher_amt_api_client.amt_api_common import AMT_ApiCommon
from mecher_amt_api_client.models.browserstack_owner import BrowserstackOwner


class BSOwner_Api(AMT_ApiCommon):
    def __init__(self):
        super().__init__()
        self.browserstack_owner_url = self.url + "/browserstack_owner"

    def get_bs_owner(self, bs_owner: str) -> BrowserstackOwner | None:
        with allure.step('AMT API: Get Browserstack Owner'):
            response = self.send_request(method='get',
                                         params={'bs_owner': bs_owner},
                                         uri=self.browserstack_owner_url)
            if response.status_code == 200:
                return BrowserstackOwner(**response)
            else:
                return



from pydantic import BaseModel

DEFAULT_APP_FORMAT = 'v0_0_1'


class AppRunConfig(BaseModel):
    name: str = 'project__mecher__ios__iphone_12__14_4__v0_0_1__v0_0_1'
    bs_owner: str = 'mecher'
    project: str = 'project'
    platform: str = 'ios'
    device: str = 'iPhone 14'
    platform_version: str = '15'
    app_version: str = DEFAULT_APP_FORMAT
    tag: str = ''
    locators_version: str = DEFAULT_APP_FORMAT
    locators_comment: str = ''

    class Config:
        openapi_example = {
            "iOS Example": {
                "summary": "Example of AppRunConfig for iOS",
                "value": {
                    "name": "project__mecher__ios__iphone_12__14_4__v0_0_1__v0_0_1",
                    "bs_owner": "mecher",
                    "project": "project",
                    "platform": "ios",
                    "device": "iPhone 13",
                    "platform_version": "15",
                    "app_version": DEFAULT_APP_FORMAT,
                    "tag": "",
                    "locators_version": DEFAULT_APP_FORMAT,
                    "locators_comment": ""
                }
            },
            "Android Example": {
                "summary": "Example of AppRunConfig for Android",
                "value": {
                    "name": "project__mecher__ios__iphone_12__14_4__v0_0_1__v0_0_1",
                    "bs_owner": "mecher",
                    "project": "project",
                    "platform": "android",
                    "device": "Google Pixel 7",
                    "platform_version": "13.0",
                    "app_version": DEFAULT_APP_FORMAT,
                    "tag": "",
                    "locators_version": DEFAULT_APP_FORMAT,
                    "locators_comment": ""
                }
            }
        }

    @staticmethod
    def default_ios_instance() -> 'AppRunConfig':
        return AppRunConfig(**AppRunConfig.Config.openapi_example['iOS Example']['value'])

    @staticmethod
    def default_android_instance() -> 'AppRunConfig':
        return AppRunConfig(**AppRunConfig.Config.openapi_example['Android Example']['value'])


class AppRunConfig_WithBSInfo(AppRunConfig):
    bs_app_url: str
    bs_app_name: str



import allure

from mecher_amt_api_client.amt_api_common import AMT_ApiCommon
from mecher_amt_api_client.models.app_run_config import AppRunConfig


class AppRunConfig_Api(AMT_ApiCommon):
    def __init__(self):
        super().__init__()
        self.app_run_config_url = self.url + "/app_run_config"

    def upload_app_run_config(self, app_run_config: AppRunConfig) -> AppRunConfig | None:
        with allure.step('AMT API: Upload App Run Config'):
            response = self.send_request(method='post',
                                         body=app_run_config.dict(),
                                         uri=self.app_run_config_url)
            if response.status_code == 200:
                return AppRunConfig(**response)
            else:
                return

    def get_app_run_config(self, app_run_config_data: AppRunConfig) -> AppRunConfig | None:
        uri = self.app_run_config_url + "/uploaded"

        with allure.step('AMT API: Get App Run Config'):
            response = self.send_request(method='post',
                                         body=app_run_config_data.dict(),
                                         uri=uri)
            if response.status_code == 200:
                return AppRunConfig(**response)
            else:
                return



import os
from setuptools import find_packages
from setuptools import setup

with open('requirements.txt') as f:
    REQUIRES = f.readlines()

setup(
    description='AMT API client',
    long_description='AMT API client',
    name='mecher_amt_api_client',
    url='https://github.com/MaximChernyak98/mecher_amt_api_client',

    maintainer='mecher',
    version=os.getenv("PACKAGE_VERSION", "0.0.1"),
    keywords=['testing'],
    install_requires=REQUIRES,
    packages=find_packages(),
    include_package_data=True,
    dependency_links=[],
    long_description_content_type='text/plain',
    setup_requires=['setuptools-git-versioning'],
)



from pathlib import Path

import allure

from mecher_amt_api_client.amt_api_common import AMT_ApiCommon
from mecher_amt_api_client.models.app_artefact import AppArtefact__Read, AppArtefact__Search


class AppArtefact_Api(AMT_ApiCommon):
    def __init__(self):
        super().__init__()
        self.app_artefact_url = self.url + "/app_artefact"

    def upload_app_artefact(self,
                            file_path: str,
                            project: str,
                            bs_owner: str = 'mecher',
                            version: str = 'v0_0_1'
                            ) -> AppArtefact__Read | None:
        with allure.step('AMT API: Upload App Artefact'):
            multipart_form_data = {
                'file': (f'{Path(file_path).name}', open(file_path, 'rb')),
            }
            params = {
                'project': project,
                'bs_owner': bs_owner,
                'version': version,
            }
            response = self.send_request(method='post',
                                         files=multipart_form_data,
                                         params=params,
                                         uri=self.app_artefact_url)
            if response.status_code == 200:
                return AppArtefact__Read(**response)
            else:
                return

    def get_app_artefact(self, app_artefact_for_search: AppArtefact__Search) -> AppArtefact__Read | None:
        uri = self.app_artefact_url + "/uploaded"

        with allure.step('AMT API: Get App Artefact'):
            response = self.send_request(method='post',
                                         body=app_artefact_for_search.dict(),
                                         uri=uri)
            if response.status_code == 200:
                return AppArtefact__Read(**response)
            else:
                return
